/*
 * Author: Dušan Kolář
 */

#ifndef PARSE_MMS
#define PARSE_MMS

typedef enum e_mms_parse_error {
    MMSPE_OK,
    MMSPE_TOO_SHORT,                  // passed buffer is shorter then smallest packet
    MMSPE_LENGTH_MISMATCH,            // lengths inside don't match
    MMSPE_OUT_OF_DATA,                // running out of the passed data length
    MMSPE_TLV_LARGE,                  // size of TLV goes more than 0xffff
    MMSPE_TLV_INDEFINITE,             // TLV length set as indefinite
    
    MMSPE_COTP_MAGIC_FAIL,            // COTP type magic expected, not found
    MMSPE_SPDU_MAGIC_FAIL,            // SPDU type magic expected, not found
    MMSPE_PPDU_MAGIC_FAIL,            // PPDU type magic expected, not found
    MMSPE_PDV_LIST_MAGIC_FAIL,        // PDV list magic expected, not found
    MMSPE_ASN_MAGIC_FAIL,             // ASN tpye magic expected, not found
    MMSPE_MMS_MAGIC_FAIL,             // Allowed MMS type magic expected, not found    
} mms_parse_error_t;


#include <inttypes.h>


typedef struct s_mms_data {
    uint8_t     mms_type_tag;         // MMS type tag
    uint8_t     service_flag;         // zero, if 1 then service type tag present
    uint8_t     service_type_tag;     // service type tag
} mms_data_t;

mms_data_t *parse_mms(uint8_t *input, uint32_t size, mms_parse_error_t *error);

#endif
