/*
 * Author: Dušan Kolář
 */

#include <stdio.h>
#include <malloc.h>

#include "parse_mms.h"

void demo_parse_packet(uint8_t *data, long packet_size) {
    puts("Parsing...");
    
    mms_parse_error_t error;
    mms_data_t *mms = parse_mms(data, (uint32_t)packet_size, &error);
    
    puts("Parsing completed...");

    if (mms == NULL) {
        puts("Packet parsing failed");
        switch (error) {
            case MMSPE_TOO_SHORT: {
                puts("TOO_SHORT");
                break;
            }
            case MMSPE_OUT_OF_DATA: {
                puts("OUT_OF_DATA");
                break;
            }
            case MMSPE_LENGTH_MISMATCH: {
                puts("LENGTH_MISMATCH");
                break;
            }
            case MMSPE_TLV_LARGE: {
                puts("TLV_LARGE");
                break;
            }
            case MMSPE_TLV_INDEFINITE: {
                puts("TLV_INDEFINITE");
                break;
            }
            case MMSPE_COTP_MAGIC_FAIL: {
                puts("COTP_MAGIC_FAIL");
                break;
            }
            case MMSPE_SPDU_MAGIC_FAIL: {
                puts("SPDU_MAGIC_FAIL");
                break;
            }
            case MMSPE_PPDU_MAGIC_FAIL: {
                puts("PPDU_MAGIC_FAIL");
                break;
            }
            case MMSPE_PDV_LIST_MAGIC_FAIL: {
                puts("PDV_LIST_MAGIC_FAIL");
                break;
            }
            case MMSPE_ASN_MAGIC_FAIL: {
                puts("ASN_MAGIC_FAIL");
                break;
            }
            case MMSPE_MMS_MAGIC_FAIL: {
                puts("MMS_MAGIC_FAIL");
                break;
            }
            default: {
                puts("Unknown reason");
                break;
            }
        }
    } else {
        printf("MMS type: %2X\n", mms->mms_type_tag);
        if (mms->service_flag) {
            printf("MMS service: %2X\n", mms->service_type_tag);
        } else {
            puts("No service");
        }
    }
    
}

int main(int argc, char *argv[]) {
    if (argc<2) {
        puts("Usage: test_mms <mms_packet_file(s)>");
        return 1;
    }
    
    for (int i=1; i<argc;++i) {
        printf("Started %s...\n",argv[i]);
    
        FILE *packet;
        if ((packet = fopen(argv[i],"rb")) == NULL) {
            printf("Cannot open file: %s\n", argv[i]);
            return 2;
        }
        
        if (fseek(packet, 0L, SEEK_END)) {
            printf("Cannot work with file: %s\n",argv[i]);
            fclose(packet);
            return 3;
        }
        
        long packet_size = ftell(packet);
        
        if (packet_size<0) {
            printf("Cannot work with file: %s\n",argv[i]);
            fclose(packet);
            return 4;
        }
        
        if (packet_size > 0x10000L) {
            printf("Packet size too large: %ld\n",packet_size);
            fclose(packet);
            return 5;
        }
        
        if (fseek(packet, 0L, SEEK_SET)) {
            printf("Cannot work with file: %s\n",argv[i]);
            fclose(packet);
            return 6;
        }
        
        uint8_t *data = malloc((size_t)packet_size);
        
        if (fread((void*)data,1,(size_t)packet_size,packet) != (size_t)packet_size) {
            printf("Cannot read file: %s\n",argv[i]);
            fclose(packet);
            return 7;
        }
        
        fclose(packet);
        
        demo_parse_packet(data, packet_size);
        
        free((void*)data);
        
        printf("Done %s !\n\n",argv[i]);
        
    } // end for
    
    return 0;
} // main

// EOF
